"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CodeBlock = void 0;
const prosemirror_commands_1 = require("prosemirror-commands");
const prosemirror_inputrules_1 = require("prosemirror-inputrules");
const prosemirror_model_1 = require("prosemirror-model");
const prosemirror_state_1 = require("prosemirror-state");
const prosemirror_utils_1 = require("prosemirror-utils");
const commands_1 = require("./commands");
const const_1 = require("./const");
const handle_paste_1 = require("./handle-paste");
const CodeBlock = (builder, opts) => {
    builder.addNode(const_1.codeBlock, () => ({
        spec: {
            attrs: { [const_1.langAttr]: { default: 'text' } },
            content: 'text*',
            group: 'block',
            code: true,
            marks: '',
            selectable: true,
            allowSelection: true,
            parseDOM: [
                {
                    tag: 'pre',
                    preserveWhitespace: 'full',
                    getAttrs: (node) => ({
                        [const_1.langAttr]: node.getAttribute(const_1.langAttr) || '',
                    }),
                },
            ],
            toDOM({ attrs }) {
                return ['pre', attrs[const_1.langAttr] ? attrs : {}, ['code', 0]];
            },
        },
        fromYfm: {
            tokenSpec: {
                name: const_1.codeBlock,
                type: 'block',
                noCloseToken: true,
            },
        },
        toYfm: (state, node) => {
            state.write('```' + (node.attrs[const_1.langAttr] || '') + '\n');
            state.text(node.textContent, false);
            state.ensureNewLine();
            state.write('```');
            state.closeBlock(node);
        },
    }));
    builder.addNode('fence', () => ({
        //  we adding this node only for define specific 'fence' parser token,
        //  which parse fence md token to code_block node
        spec: {},
        fromYfm: {
            tokenSpec: {
                name: const_1.codeBlock,
                type: 'block',
                noCloseToken: true,
                getAttrs: (tok) => ({ [const_1.langAttr]: tok.info || '' }),
            },
        },
        toYfm: () => {
            throw new Error('Unexpected toYfm() call on fence node');
        },
    }));
    builder.addKeymap((deps) => {
        const { codeBlockKey } = opts;
        const bindings = { Backspace: commands_1.resetCodeblock };
        if (codeBlockKey)
            bindings[codeBlockKey] = (0, prosemirror_commands_1.setBlockType)((0, const_1.cbType)(deps.schema));
        return bindings;
    });
    builder.addInputRules(({ schema }) => ({ rules: [codeBlockRule((0, const_1.cbType)(schema))] }));
    builder.addAction(const_1.cbAction, ({ schema }) => {
        const cb = (0, const_1.cbType)(schema);
        const cmd = (0, prosemirror_commands_1.setBlockType)(cb);
        return {
            isActive: (state) => (0, prosemirror_utils_1.hasParentNodeOfType)(cb)(state.selection),
            isEnable: cmd,
            run: cmd,
        };
    });
    builder.addPlugin(() => new prosemirror_state_1.Plugin({
        props: {
            handleDOMEvents: {
                paste(view, e) {
                    if ((0, handle_paste_1.handlePaste)(view, e, prosemirror_model_1.Slice.empty)) {
                        e.preventDefault();
                        return true;
                    }
                    return false;
                },
            },
        },
    }), builder.Priority.High);
};
exports.CodeBlock = CodeBlock;
// Given a code block node type, returns an input rule that turns a
// textblock starting with three backticks into a code block.
function codeBlockRule(nodeType) {
    return (0, prosemirror_inputrules_1.textblockTypeInputRule)(/^```$/, nodeType);
}
