"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.YfmNote = exports.noteTitleType = exports.noteType = void 0;
const tslib_1 = require("tslib");
const log_1 = tslib_1.__importDefault(require("@doc-tools/transform/lib/log"));
const notes_1 = tslib_1.__importDefault(require("@doc-tools/transform/lib/plugins/notes"));
const prosemirror_commands_1 = require("prosemirror-commands");
const toYfm_1 = require("./toYfm");
const const_1 = require("./const");
const fromYfm_1 = require("./fromYfm");
const spec_1 = require("./spec");
const toYfmNote_1 = require("./actions/toYfmNote");
const inputrules_1 = require("../../../utils/inputrules");
const commands_1 = require("./commands");
const utils_1 = require("./utils");
const noteAction = 'toYfmNote';
var utils_2 = require("./utils");
Object.defineProperty(exports, "noteType", { enumerable: true, get: function () { return utils_2.noteType; } });
Object.defineProperty(exports, "noteTitleType", { enumerable: true, get: function () { return utils_2.noteTitleType; } });
const YfmNote = (builder, opts) => {
    const spec = (0, spec_1.getSpec)(opts);
    builder
        .configureMd((md) => md.use(notes_1.default, { log: log_1.default }))
        .addNode(const_1.NoteNode.Note, () => ({
        spec: spec[const_1.NoteNode.Note],
        toYfm: toYfm_1.toYfm[const_1.NoteNode.Note],
        fromYfm: {
            tokenSpec: fromYfm_1.fromYfm[const_1.NoteNode.Note],
        },
    }))
        .addNode(const_1.NoteNode.NoteTitle, () => ({
        spec: spec[const_1.NoteNode.NoteTitle],
        toYfm: toYfm_1.toYfm[const_1.NoteNode.NoteTitle],
        fromYfm: {
            tokenSpec: fromYfm_1.fromYfm[const_1.NoteNode.NoteTitle],
        },
    }))
        .addKeymap(() => ({
        Enter: commands_1.exitFromNoteTitle,
        Backspace: (0, prosemirror_commands_1.chainCommands)(commands_1.backToNoteTitle, commands_1.removeNote),
    }))
        .addAction(noteAction, () => toYfmNote_1.toYfmNote)
        .addInputRules(({ schema }) => ({
        rules: [(0, inputrules_1.nodeInputRule)(/(?:^)({% note)\s$/, (0, utils_1.noteType)(schema).createAndFill(), 1)],
    }));
    if (opts === null || opts === void 0 ? void 0 : opts.yfmNoteKey) {
        const { yfmNoteKey } = opts;
        builder.addKeymap(() => ({ [yfmNoteKey]: toYfmNote_1.createYfmNote }));
    }
};
exports.YfmNote = YfmNote;
