import { toggleMark } from 'prosemirror-commands';
import { createToggleMarkAction } from '../../../utils/actions';
import { markTypeFactory } from '../../../utils/schema';
import { markInputRule } from '../../../utils/inputrules';
export const bold = 'strong';
const bAction = 'bold';
const bType = markTypeFactory(bold);
export const Bold = (builder, opts) => {
    builder
        .addMark(bold, () => ({
        spec: {
            parseDOM: [
                { tag: 'b' },
                { tag: 'strong' },
                {
                    style: 'font-weight',
                    getAttrs: (value) => /^(bold(er)?|[5-9]\d{2,})$/.test(value) && null,
                },
            ],
            toDOM() {
                return ['strong'];
            },
        },
        fromYfm: {
            tokenSpec: {
                name: bold,
                type: 'mark',
            },
        },
        toYfm: { open: '**', close: '**', mixable: true, expelEnclosingWhitespace: true },
    }))
        .addAction(bAction, ({ schema }) => createToggleMarkAction(bType(schema)));
    if (opts === null || opts === void 0 ? void 0 : opts.boldKey) {
        const { boldKey } = opts;
        builder.addKeymap(({ schema }) => ({ [boldKey]: toggleMark(bType(schema)) }));
    }
    builder.addInputRules(({ schema }) => ({
        rules: [
            markInputRule({ open: '**', close: '**', ignoreBetween: '*' }, bType(schema)),
            markInputRule({ open: '__', close: '__', ignoreBetween: '_' }, bType(schema)),
        ],
    }));
};
